(* File generated from fpp.idl *)

type internal

(*
 This file is part of the APRON Library, released under LGPL license.
 Please read the COPYING file packaged in the distribution.
*)

 
(** Fppol abstract domain. *)
 


 
type t
(** Type of fppols.

Fppols are defined by conjunctions of inequalities of the form
[+/-x_i +/- x_j >= 0].

Abstract values which are fppols have the type [t Apron.AbstractX.t].

Managers allocated for fppols have the type [t Apron.manager.t].
*)

 
(** Allocate a new manager to manipulate fppols. *)
external manager_alloc : unit -> t Apron.Manager.t
	= "camlidl_fpp_fpp_manager_alloc"

(** No internal parameters for now... *)
external manager_get_internal : t Apron.Manager.t -> internal
	= "camlidl_fpp_manager_get_internal"

(** Approximate a set of generators to an abstract value, with best precision. *)
external of_generator_array : t Apron.Manager.t -> int -> int -> Apron.Generator0.t array -> t Apron.Abstract0.t
	= "camlidl_fpp_ap_abstract0_fpp_of_generator_array"

(** Widening with scalar thresholds. *)
external widening_thresholds : t Apron.Manager.t -> t Apron.Abstract0.t -> t Apron.Abstract0.t -> Apron.Scalar.t array -> t Apron.Abstract0.t
	= "camlidl_fpp_ap_abstract0_fpp_widening_thresholds"

(** Standard narrowing. *)
external narrowing : t Apron.Manager.t -> t Apron.Abstract0.t -> t Apron.Abstract0.t -> t Apron.Abstract0.t
	= "camlidl_fpp_ap_abstract0_fpp_narrowing"

(** Perturbation. *)
external add_epsilon : t Apron.Manager.t -> t Apron.Abstract0.t -> Apron.Scalar.t -> t Apron.Abstract0.t
	= "camlidl_fpp_ap_abstract0_fpp_add_epsilon"

(** Perturbation. *)
external add_epsilon_bin : t Apron.Manager.t -> t Apron.Abstract0.t -> t Apron.Abstract0.t -> Apron.Scalar.t -> t Apron.Abstract0.t
	= "camlidl_fpp_ap_abstract0_fpp_add_epsilon_bin"

(** Algorithms. *)
val pre_widening : int

(** {2 Type conversions} *)

val manager_is_fpp : 'a Apron.Manager.t -> bool
  (** Return [true] iff the argument manager is a fppol manager *)
val manager_of_fpp : t Apron.Manager.t -> 'a Apron.Manager.t
  (** Make a fppol manager generic *)
val manager_to_fpp : 'a Apron.Manager.t -> t Apron.Manager.t
  (** Instanciate the type of a fppol manager.
      Raises [Failure] if the argument manager is not a fppol manager *)
module Abstract0 : sig
  val is_fpp : 'a Apron.Abstract0.t -> bool
    (** Return [true] iff the argument value is a fpp value *)
  val of_fpp : t Apron.Abstract0.t -> 'a Apron.Abstract0.t
    (** Make an a fpp value generic *)
  val to_fpp : 'a Apron.Abstract0.t -> t Apron.Abstract0.t
    (** Instanciate the type of a fpp value.
	Raises [Failure] if the argument value is not a fpp value *)
end
module Abstract1 : sig
  val is_fpp : 'a Apron.Abstract1.t -> bool
    (** Return [true] iff the argument value is a fpp value *)
  val of_fpp : t Apron.Abstract1.t -> 'a Apron.Abstract1.t
    (** Make a fpp value generic *)
  val to_fpp : 'a Apron.Abstract1.t -> t Apron.Abstract1.t
    (** Instanciate the type of a fpp value.
	Raises [Failure] if the argument value is not a fpp value *)
end


(**
{2 Compilation information}

See {!Introduction.compilation} for complete explanations.
We just show examples with the file [mlexample.ml].

{3 Bytecode compilation}

{[ocamlc -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -o mlexample.byte \
  bigarray.cma gmp.cma apron.cma fppD.cma mlexample.ml]}

{[ocamlc -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -make-runtime -o myrun \
  bigarray.cma gmp.cma apron.cma fppD.cma

ocamlc -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -use-runtime myrun -o mlexample.byte \
  bigarray.cma gmp.cma apron.cma fppD.cma mlexample.ml ]}

{3 Native-code compilation}

{[ocamlopt -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -o mlexample.opt \
  bigarray.cmxa gmp.cmxa apron.cmxa fppD.cmxa mlexample.ml ]}

{3 Without auto-linking feature}

{[ocamlopt -I $MLGMPIDL_PREFIX/lib -I $APRON_PREFIX/lib -noautolink -o mlexample.opt \
  bigarray.cmxa gmp.cmxa apron.cmxa fppD.cmxa mlexample.ml \
  -cclib "-L$MLGMPIDL_PREFIX/lib -L$APRON_PREFIX/lib \
	  -lfppD_caml_debug -lfppD_debug \
	  -lapron_caml_debug -lapron_debug \
	  -lgmp_caml -L$MPFR_PREFIX/lib -lmpfr -L$GMP_PREFIX/lib -lgmp \
	  -lglpk -L$GLPK_PREFIX/lib \	  
	  -L$CAMLIDL_PREFIX/lib/ocaml -lcamlidl \
	  -lbigarray" ]}

*)
